/*
 * multican 0.0.5, USB remote control tool for Canon cameras.
 *
 * Copyright (C) 2006 Jindrich Novy (jnovy@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CANONIO_H_INCLUDED
#define CANONIO_H_INCLUDED

#define _GNU_SOURCE

#include "usbio.h"

/*#define DEBUG 1*/
#define ABSOLUTE_TIME 1

#define THUMB_TO_PC	0x0001
#define FULL_TO_PC	0x0002
#define THUMB_TO_DRIVE	0x0004
#define FULL_TO_DRIVE	0x0008

#define FLASH_OFF  0x00
#define FLASH_ON   0x01
#define FLASH_AUTO 0x02

#ifdef DEBUG
#  define FUNCSTART fprintf(stderr, "%20s(): % 4d --------------------------------------------\n", __FUNCTION__, __LINE__);
#else
#  define FUNCSTART
#endif

typedef struct {
	char *token;
	char *code;
} rel_params;

static rel_params quality_vals[] = {
	{ "fine-l",		"\x03\x01\x00" },
	{ "fine-m",		"\x03\x01\x01" },
	{ "fine-s",		"\x03\x01\x02" },
	{ "raw-fine-l",		"\x34\x12\x00" },
	{ "raw-fine-m",		"\x34\x12\x01" },
	{ "raw-fine-s",		"\x34\x12\x02" },
	{ "coarse-l",		"\x02\x01\x00" },
	{ "coarse-m",		"\x02\x01\x01" },
	{ "coarse-s",		"\x02\x01\x02" },
	{ "raw-coarse-l",	"\x24\x12\x00" },
	{ "raw-coarse-m",	"\x24\x12\x01" },
	{ "raw-coarse-s",	"\x24\x12\x02" },
	{ "raw",		"\x04\x02\x00" },
	{ NULL, 		NULL }
};

static rel_params sequence_vals[] = {
	{ "single",	"\x00\x00\x00" },
	{ "multi",	"\x00\x00\x01" },
	{ "self-timer",	"\x64\x00\x00" },
	{ NULL, 	NULL }
};

static rel_params beep_vals[] = {
	{ "off",	"\x00" },
	{ "on",		"\x01" },
	{ NULL, 	NULL }
};

static rel_params mode_vals[] = {
	{ "full-auto",	"\x00\x00" },
	{ "landscape",	"\x00\x02" },
	{ "night",	"\x00\x05" },
	{ "portrait",	"\x00\x08" },
	{ "sports",	"\x00\x09" },
	{ "macro",	"\x00\x0a" },
	{ "no-flash",	"\x00\x0f" },
	{ "p",		"\x01\x01" },
	{ "tv",		"\x02\x01" },
	{ "av",		"\x03\x01" },
	{ "m",		"\x04\x01" },
	{ "a-dep",	"\x05\x01" },
	{ "b",		"\x07\x01" },
	{ NULL,		NULL }
};

static rel_params autoexp_vals[] = {
	{ "spot",	"\x01" },
	{ "middle-spot","\x03" },
	{ "middle",	"\x04" },
	{ "integral",	"\x05" },
	{ NULL,		NULL }
};

static rel_params whitebal_vals[] = {
	{ "auto",	"\x00" },
	{ "sunny",	"\x01" },
	{ "cloudy",	"\x02" },
	{ "art-light",	"\x03" },
	{ "flour-light","\x04" },
	{ "flash",	"\x05" },
	{ "manual",	"\x06" },
	{ "shade",	"\x08" },
	{ "color-temp",	"\x09" },	
	{ NULL,		NULL }
};

static rel_params autofocus_vals[] = {
	{ "af-one-shot",	"\x00" },
	{ "ai-servo",		"\x01" },
	{ "ai-focus",		"\x02" },
	{ "mf",			"\x03" },
	{ NULL,			NULL}
};

static rel_params tv_vals[] = {
	{ "bulb",	"\x04" },
	{ "30\"",	"\x10" },
	{ "25\"",	"\x13" },
	{ "20\"",	"\x15" },
	{ "15\"",	"\x18" },
	{ "13\"",	"\x1b" },
	{ "10\"",	"\x1d" },
	{ "8\"",	"\x20" },
	{ "6\"",	"\x23" },
	{ "5\"",	"\x25" },
	{ "4\"",	"\x28" },
	{ "3\"2",	"\x2b" },
	{ "2\"5",	"\x2d" },
	{ "2\"",	"\x30" },
	{ "1\"6",	"\x33" },
	{ "1\"3",	"\x35" },
	{ "1\"",	"\x38" },
	{ "0\"8",	"\x3b" },
	{ "0\"6",	"\x3d" },
	{ "0\"5",	"\x40" },
	{ "0\"4",	"\x43" },
	{ "0\"3",	"\x45" },
	{ "4",		"\x48" },
	{ "5",		"\x4b" },
	{ "6",		"\x4d" },
	{ "8",		"\x50" },
	{ "10",		"\x53" },
	{ "13",		"\x55" },
	{ "15", 	"\x58" },
	{ "20", 	"\x5b" },
	{ "25", 	"\x5d" },
	{ "30", 	"\x60" },
	{ "40", 	"\x63" },
	{ "50", 	"\x65" },
	{ "60", 	"\x68" },
	{ "80",		"\x6b" },
	{ "100",	"\x6d" },
	{ "125",	"\x70" },
	{ "160",	"\x73" },
	{ "200",	"\x75" },
	{ "250",	"\x78" },
	{ "320",	"\x7b" },
	{ "400",	"\x7d" },
	{ "500",	"\x80" },
	{ "640",	"\x83" },
	{ "800",	"\x85" },
	{ "1000",	"\x88" },
	{ "1250",	"\x8b" },
	{ "1600",	"\x8d" },
	{ "2000",	"\x90" },
	{ "2500",	"\x93" },
	{ "3200",	"\x95" },
	{ "4000",	"\x98" },
	{ "5000",	"\x9b" },
	{ "6400",	"\x9d" },
	{ "8000",	"\xa0" },
	{ NULL,		NULL }
};

static rel_params av_vals[] = {
	{ "29",		"\x55" },
	{ "25",		"\x53" },
	{ "22",		"\x50" },
	{ "20",		"\x4d" },
	{ "18",		"\x4b" },
	{ "16",		"\x48" },
	{ "14",		"\x45" },
	{ "13",		"\x43" },
	{ "11",		"\x40" },
	{ "10",		"\x3d" },
	{ "9.0",	"\x3b" },
	{ "8.0",	"\x38" },
	{ "7.1",	"\x35" },
	{ "6.3",	"\x33" },
	{ "5.6",	"\x30" },
	{ "5.0",	"\x2d" },
	{ "4.5",	"\x2b" },
	{ "4.0",	"\x28" },
	{ "3.5",	"\x25" },
	{ "3.2",	"\x23" },
	{ "2.8",	"\x20" },
	{ NULL, 	NULL }
};

static rel_params iso_vals[] = {
	{ "3200",	"\x70" },
	{ "1600",	"\x68" },
	{ "1250",	"\x65" },
	{ "1000",	"\x63" },
	{ "800",	"\x60" },
	{ "640",	"\x5d" },
	{ "500",	"\x5b" },
	{ "400",	"\x58" },
	{ "320",	"\x55" },
	{ "250",	"\x53" },
	{ "200",	"\x50" },
	{ "160",	"\x4d" },
	{ "125",	"\x4b" },
	{ "100",	"\x48" },
	{ "50",		"\x40" },
	{ NULL,		NULL }
};

static rel_params expcorr_vals[] = {
	{ "-2.0",	"\xf0" },
	{ "-1.6",	"\xf3" },
	{ "-1.5",	"\xf4" },
	{ "-1.3",	"\xf5" },
	{ "-1.0",	"\xf8" },
	{ "-0.6",	"\xfb" },
	{ "-0.5",	"\xfc" },
	{ "-0.3",	"\xfd" },
	{ "0.0",	"\x00" },
	{ "+0.3",	"\x03" },
	{ "+0.5",	"\x04" },
	{ "+0.6",	"\x05" },
	{ "+1.0",	"\x08" },
	{ "+1.3",	"\x0b" },
	{ "+1.5",	"\x0c" },
	{ "+1.6",	"\x0d" },
	{ "+2.0",	"\x10" },
	{ NULL,		NULL }
};

static struct {
	const char *name;
	const char *offset;
	rel_params *table;
} rel_param_entries[] = {
	{ "quality",		"\x01\x02\x03",	quality_vals },
	{ "sequence",		"\x04\x05\x0a",	sequence_vals },
	{ "beep",		"\x07",		beep_vals },
	{ "mode",		"\x08\x09",	mode_vals },
	{ "auto-exp-mode",	"\x0c",		autoexp_vals },
	{ "white-balance",	"\x10",		whitebal_vals },
	{ "auto-focus",		"\x12",		autofocus_vals },
	{ "iso",		"\x1a",		iso_vals },
	{ "av",			"\x1c",		av_vals },
	{ "tv",			"\x1e",		tv_vals },
	{ "exp-corr",		"\x20",		expcorr_vals },
	{ NULL,			NULL,		NULL }
};

int  canon_scan( camera **c );
int  canon_init_camera( camera *c );
int  canon_init_all( camera *c );
void canon_identify_camera( camera *c );
void canon_power_status( camera *c );
void canon_eos_id( camera *c );
void canon_init_control( camera *c );
void canon_flash( camera *c, unsigned char flags );
void canon_transfer( camera *c, unsigned char flags );
void canon_get_zoom( camera *c );
void canon_avail_shots( camera *c );
void canon_exit_control( camera *c );
void canon_close( camera **cptr );
void canon_set_owner( camera *c, char *name );
void canon_get_rel_params( camera *c );
void canon_set_rel_params( camera *c );
void canon_rel( camera *c, char *param, char *token );
void canon_release_shutter( camera *c );

#endif
